@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2014 The Squid Software Foundation and contributors
+ * Copyright (C) 1996-2022 The Squid Software Foundation and contributors
  *
  * Squid software is distributed under GPLv2+ license and includes
  * contributions from numerous individuals and organizations.
@@ -9,18 +9,18 @@
 /* DEBUG: none          Generate squid.conf.default and cf_parser.cci */
 
 /*****************************************************************************
- * Abstract:	This program parses the input file and generates code and
- *		files used to configure the variables in squid.
- *		(ie it creates the squid.conf.default file from the cf.data file)
+ * Abstract:    This program parses the input file and generates code and
+ *      files used to configure the variables in squid.
+ *      (ie it creates the squid.conf.default file from the cf.data file)
  *
- *		The output files are as follows:
- *		cf_parser.cci - this file contains, default_all() which
- *			  initializes variables with the default
- *			  values, parse_line() that parses line from
- *			  squid.conf.default, dump_config that dumps the
- *			  current the values of the variables.
- *		squid.conf.default - default configuration file given to the server
- *			 administrator.
+ *      The output files are as follows:
+ *      cf_parser.cci - this file contains, default_all() which
+ *            initializes variables with the default
+ *            values, parse_line() that parses line from
+ *            squid.conf.default, dump_config that dumps the
+ *            current the values of the variables.
+ *      squid.conf.default - default configuration file given to the server
+ *           administrator.
  *****************************************************************************/
 
 /*
@@ -44,11 +44,11 @@ _FILE_OFFSET_BITS==64
 
 #include "cf_gen_defines.cci"
 
-#define MAX_LINE	1024	/* longest configuration line */
-#define _PATH_PARSER		"cf_parser.cci"
-#define _PATH_SQUID_CONF	"squid.conf.documented"
-#define _PATH_SQUID_CONF_SHORT	"squid.conf.default"
-#define _PATH_CF_DEPEND		"cf.data.depend"
+#define MAX_LINE    1024    /* longest configuration line */
+#define _PATH_PARSER        "cf_parser.cci"
+#define _PATH_SQUID_CONF    "squid.conf.documented"
+#define _PATH_SQUID_CONF_SHORT  "squid.conf.default"
+#define _PATH_CF_DEPEND     "cf.data.depend"
 
 enum State {
     sSTART,
@@ -88,9 +88,9 @@ class Entry
 {
 public:
     Entry(const char *str) :
-            name(str), alias(),type(), loc(),
-            defaults(), comment(), ifdef(), doc(), nocomment(),
-            array_flag(0) {}
+        name(str), alias(),type(), loc(),
+        defaults(), comment(), ifdef(), doc(), nocomment(),
+        array_flag(0) {}
     ~Entry() {}
 
     std::string name;
@@ -150,20 +150,20 @@ checkDepend(const std::string &directive, const char *name, const TypeList &type
             }
             if (entry == entries.end()) {
                 std::cerr << "ERROR: '" << directive << "' (" << name << ") depends on '" << *dep << "'\n";
-                exit(1);
+                exit(EXIT_FAILURE);
             }
         }
         return;
     }
     std::cerr << "ERROR: Dependencies for cf.data type '" << name << "' used in ' " << directive << "' not defined\n" ;
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 static void
 usage(const char *program_name)
 {
     std::cerr << "Usage: " << program_name << " cf.data cf.data.depend\n";
-    exit(1);
+    exit(EXIT_FAILURE);
 }
 
 static void
@@ -204,7 +204,7 @@ main(int argc, char *argv[])
     if (fp.fail()) {
         std::cerr << "Error while opening type dependencies file '" <<
                   type_depend << "': " << strerror(errno) << std::endl;
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     while (fp.good()) {
@@ -231,7 +231,7 @@ main(int argc, char *argv[])
     if (fp.fail()) {
         std::cerr << "Error while opening input file '" <<
                   input_filename << "': " << strerror(errno) << std::endl;
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     state = sSTART;
@@ -247,14 +247,14 @@ main(int argc, char *argv[])
         if (strncmp(buff, "IF ", 3) == 0) {
             if ((ptr = strtok(buff + 3, WS)) == NULL) {
                 errorMsg(input_filename, linenum, "Missing IF parameter");
-                exit(1);
+                exit(EXIT_FAILURE);
             }
             IFDEFS.push(ptr);
             continue;
         } else if (strcmp(buff, "ENDIF") == 0) {
             if (IFDEFS.size() == 0) {
                 errorMsg(input_filename, linenum, "ENDIF without IF first");
-                exit(1);
+                exit(EXIT_FAILURE);
             }
             IFDEFS.pop();
         } else if (!IFDEFS.size() || isDefined(IFDEFS.top()))
@@ -270,7 +270,7 @@ main(int argc, char *argv[])
 
                     if ((name = strtok(buff + 5, WS)) == NULL) {
                         errorMsg(input_filename, linenum, buff);
-                        exit(1);
+                        exit(EXIT_FAILURE);
                     }
 
                     entries.push_back(name);
@@ -287,7 +287,7 @@ main(int argc, char *argv[])
                     state = sDOC;
                 } else {
                     errorMsg(input_filename, linenum, buff);
-                    exit(1);
+                    exit(EXIT_FAILURE);
                 }
 
                 break;
@@ -336,14 +336,14 @@ main(int argc, char *argv[])
                 } else if (!strncmp(buff, "LOC:", 4)) {
                     if ((ptr = strtok(buff + 4, WS)) == NULL) {
                         errorMsg(input_filename, linenum, buff);
-                        exit(1);
+                        exit(EXIT_FAILURE);
                     }
 
                     curr.loc = ptr;
                 } else if (!strncmp(buff, "TYPE:", 5)) {
                     if ((ptr = strtok(buff + 5, WS)) == NULL) {
                         errorMsg(input_filename, linenum, buff);
-                        exit(1);
+                        exit(EXIT_FAILURE);
                     }
 
                     /* hack to support arrays, rather than pointers */
@@ -357,7 +357,7 @@ main(int argc, char *argv[])
                 } else if (!strncmp(buff, "IFDEF:", 6)) {
                     if ((ptr = strtok(buff + 6, WS)) == NULL) {
                         errorMsg(input_filename, linenum, buff);
-                        exit(1);
+                        exit(EXIT_FAILURE);
                     }
 
                     curr.ifdef = ptr;
@@ -367,7 +367,7 @@ main(int argc, char *argv[])
                     state = sSTART;
                 } else {
                     errorMsg(input_filename, linenum, buff);
-                    exit(1);
+                    exit(EXIT_FAILURE);
                 }
             }
             break;
@@ -391,17 +391,18 @@ main(int argc, char *argv[])
                     entries.back().nocomment.push_back(buff);
                 }
                 break;
-
+#if 0
             case sEXIT:
-                assert(0);		/* should never get here */
+                assert(0);      /* should never get here */
                 break;
+#endif
             }
 
     }
 
     if (state != sEXIT) {
         errorMsg(input_filename, linenum, "Error: unexpected EOF");
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     fp.close();
@@ -420,17 +421,17 @@ main(int argc, char *argv[])
     if (!fout.good()) {
         std::cerr << "Error while opening output .c file '" <<
                   output_filename << "': " << strerror(errno) << std::endl;
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     fout <<  "/*\n" <<
-    " * Generated automatically from " << input_filename << " by " <<
-    argv[0] << "\n"
-    " *\n"
-    " * Abstract: This file contains routines used to configure the\n"
-    " *           variables in the squid server.\n"
-    " */\n"
-    "\n";
+         " * Generated automatically from " << input_filename << " by " <<
+         argv[0] << "\n"
+         " *\n"
+         " * Abstract: This file contains routines used to configure the\n"
+         " *           variables in the squid server.\n"
+         " */\n"
+         "\n";
 
     rc = gen_default(entries, fout);
 
@@ -451,7 +452,7 @@ main(int argc, char *argv[])
     if (!fout.good()) {
         std::cerr << "Error while opening output conf file '" <<
                   output_filename << "': " << strerror(errno) << std::endl;
-        exit(1);
+        exit(EXIT_FAILURE);
     }
 
     gen_conf(entries, fout, 1);
@@ -462,7 +463,7 @@ main(int argc, char *argv[])
     if (!fout.good()) {
         std::cerr << "Error while opening output short conf file '" <<
                   output_filename << "': " << strerror(errno) << std::endl;
-        exit(1);
+        exit(EXIT_FAILURE);
     }
     gen_conf(entries, fout, 0);
     fout.close();
@@ -475,19 +476,21 @@ gen_default(const EntryList &head, std::ostream &fout)
 {
     int rc = 0;
     fout << "static void" << std::endl <<
-    "default_line(const char *s)" << std::endl <<
-    "{" << std::endl <<
-    "    LOCAL_ARRAY(char, tmp_line, BUFSIZ);" << std::endl <<
-    "    xstrncpy(tmp_line, s, BUFSIZ);" << std::endl <<
-    "    xstrncpy(config_input_line, s, BUFSIZ);" << std::endl <<
-    "    config_lineno++;" << std::endl <<
-    "    parse_line(tmp_line);" << std::endl <<
-    "}" << std::endl << std::endl;
+         "default_line(const char *s)" << std::endl <<
+         "{" << std::endl <<
+         "    char *tmp_line = xstrdup(s);" << std::endl <<
+         "    int len = strlen(tmp_line);" << std::endl <<
+         "    ProcessMacros(tmp_line, len);" << std::endl <<
+         "    xstrncpy(config_input_line, tmp_line, sizeof(config_input_line));" << std::endl <<
+         "    config_lineno++;" << std::endl <<
+         "    parse_line(tmp_line);" << std::endl <<
+         "    xfree(tmp_line);" << std::endl <<
+         "}" << std::endl << std::endl;
     fout << "static void" << std::endl <<
-    "default_all(void)" << std::endl <<
-    "{" << std::endl <<
-    "    cfg_filename = \"Default Configuration\";" << std::endl <<
-    "    config_lineno = 0;" << std::endl;
+         "default_all(void)" << std::endl <<
+         "{" << std::endl <<
+         "    cfg_filename = \"Default Configuration\";" << std::endl <<
+         "    config_lineno = 0;" << std::endl;
 
     for (EntryList::const_iterator entry = head.begin(); entry != head.end(); ++entry) {
         assert(entry->name.size());
@@ -526,7 +529,7 @@ gen_default(const EntryList &head, std::ostream &fout)
     }
 
     fout << "    cfg_filename = NULL;" << std::endl <<
-    "}" << std::endl << std::endl;
+         "}" << std::endl << std::endl;
     return rc;
 }
 
@@ -534,10 +537,10 @@ static void
 gen_default_if_none(const EntryList &head, std::ostream &fout)
 {
     fout << "static void" << std::endl <<
-    "defaults_if_none(void)" << std::endl <<
-    "{" << std::endl <<
-    "    cfg_filename = \"Default Configuration (if absent)\";" << std::endl <<
-    "    config_lineno = 0;" << std::endl;
+         "defaults_if_none(void)" << std::endl <<
+         "{" << std::endl <<
+         "    cfg_filename = \"Default Configuration (if absent)\";" << std::endl <<
+         "    config_lineno = 0;" << std::endl;
 
     for (EntryList::const_iterator entry = head.begin(); entry != head.end(); ++entry) {
         assert(entry->name.size());
@@ -550,7 +553,7 @@ gen_default_if_none(const EntryList &head, std::ostream &fout)
 
         if (!entry->defaults.preset.empty()) {
             std::cerr << "ERROR: " << entry->name << " has preset defaults. DEFAULT_IF_NONE cannot be true." << std::endl;
-            exit(1);
+            exit(EXIT_FAILURE);
         }
 
         if (entry->ifdef.size())
@@ -566,7 +569,7 @@ gen_default_if_none(const EntryList &head, std::ostream &fout)
     }
 
     fout << "    cfg_filename = NULL;" << std::endl <<
-    "}" << std::endl << std::endl;
+         "}" << std::endl << std::endl;
 }
 
 /// append configuration options specified by POSTSCRIPTUM lines
@@ -574,10 +577,10 @@ static void
 gen_default_postscriptum(const EntryList &head, std::ostream &fout)
 {
     fout << "static void" << std::endl <<
-    "defaults_postscriptum(void)" << std::endl <<
-    "{" << std::endl <<
-    "    cfg_filename = \"Default Configuration (postscriptum)\";" << std::endl <<
-    "    config_lineno = 0;" << std::endl;
+         "defaults_postscriptum(void)" << std::endl <<
+         "{" << std::endl <<
+         "    cfg_filename = \"Default Configuration (postscriptum)\";" << std::endl <<
+         "    config_lineno = 0;" << std::endl;
 
     for (EntryList::const_iterator entry = head.begin(); entry != head.end(); ++entry) {
         assert(entry->name.size());
@@ -599,7 +602,7 @@ gen_default_postscriptum(const EntryList &head, std::ostream &fout)
     }
 
     fout << "    cfg_filename = NULL;" << std::endl <<
-    "}" << std::endl << std::endl;
+         "}" << std::endl << std::endl;
 }
 
 void
@@ -626,9 +629,9 @@ Entry::genParseAlias(const std::string &aName, std::ostream &fout) const
     fout << "        cfg_directive = NULL;" << std::endl;
     if (ifdef.size()) {
         fout <<
-        "#else" << std::endl <<
-        "    debugs(0, DBG_PARSE_NOTE(DBG_IMPORTANT), \"ERROR: '" << name << "' requires " << available_if(ifdef) << "\");" << std::endl <<
-        "#endif" << std::endl;
+             "#else" << std::endl <<
+             "    debugs(0, DBG_PARSE_NOTE(DBG_IMPORTANT), \"ERROR: '" << name << "' requires " << available_if(ifdef) << "\");" << std::endl <<
+             "#endif" << std::endl;
     }
     fout << "        return 1;" << std::endl;
     fout << "    };" << std::endl;
@@ -653,19 +656,19 @@ static void
 gen_parse(const EntryList &head, std::ostream &fout)
 {
     fout <<
-    "static int\n"
-    "parse_line(char *buff)\n"
-    "{\n"
-    "\tchar\t*token;\n"
-    "\tif ((token = strtok(buff, w_space)) == NULL) \n"
-    "\t\treturn 1;\t/* ignore empty lines */\n"
-    "\tConfigParser::SetCfgLine(strtok(NULL, \"\"));\n";
+         "static int\n"
+         "parse_line(char *buff)\n"
+         "{\n"
+         "\tchar\t*token;\n"
+         "\tif ((token = strtok(buff, w_space)) == NULL) \n"
+         "\t\treturn 1;\t/* ignore empty lines */\n"
+         "\tConfigParser::SetCfgLine(strtok(NULL, \"\"));\n";
 
     for (EntryList::const_iterator e = head.begin(); e != head.end(); ++e)
         e->genParse(fout);
 
     fout << "\treturn 0; /* failure */\n"
-    "}\n\n";
+         "}\n\n";
 
 }
 
@@ -673,10 +676,10 @@ static void
 gen_dump(const EntryList &head, std::ostream &fout)
 {
     fout <<
-    "static void" << std::endl <<
-    "dump_config(StoreEntry *entry)" << std::endl <<
-    "{" << std::endl <<
-    "    debugs(5, 4, HERE);" << std::endl;
+         "static void" << std::endl <<
+         "dump_config(StoreEntry *entry)" << std::endl <<
+         "{" << std::endl <<
+         "    debugs(5, 4, HERE);" << std::endl;
 
     for (EntryList::const_iterator e = head.begin(); e != head.end(); ++e) {
 
@@ -702,10 +705,10 @@ static void
 gen_free(const EntryList &head, std::ostream &fout)
 {
     fout <<
-    "static void" << std::endl <<
-    "free_all(void)" << std::endl <<
-    "{" << std::endl <<
-    "    debugs(5, 4, HERE);" << std::endl;
+         "static void" << std::endl <<
+         "free_all(void)" << std::endl <<
+         "{" << std::endl <<
+         "    debugs(5, 4, HERE);" << std::endl;
 
     for (EntryList::const_iterator e = head.begin(); e != head.end(); ++e) {
         if (!e->loc.size() || e->loc.compare("none") == 0)
@@ -779,8 +782,8 @@ gen_conf(const EntryList &head, std::ostream &fout, bool verbose_output)
         if (!isDefined(entry->ifdef)) {
             if (verbose_output) {
                 fout << "# Note: This option is only available if Squid is rebuilt with the" << std::endl <<
-                "#       " << available_if(entry->ifdef) << std::endl <<
-                "#" << std::endl;
+                     "#       " << available_if(entry->ifdef) << std::endl <<
+                     "#" << std::endl;
             }
             enabled = 0;
         }
@@ -860,3 +863,4 @@ gen_quote_escape(const std::string &var)
 
     return esc.c_str();
 }
+
